%---------------------------------------------------------------------
% Simple model to compare government spending multipliers at the ZLB
%---------------------------------------------------------------------

% Folder cleanup
delete *dynamic* *static* *results* *auxiliary_variables* *.log

clear
close all
set(0,'DefaultLineLineWidth',2)

cd ..
addpath Dynare_4.3.1
cd(curdir)
addpath toolkit_files
addpath helper_files

addpath toolkit_files
addpath helper_files

baseyear=11;

% Loops over four models, solves all of them and makes chart
for model=[1:4]
    
    
    % Solves model without learning
    if model==1
        LEARNING=0; STD_AP=0.00049; STD_AT=0.001; RHO_T=0.001; RHO_P=0.999;ERISK=0; xxx=0;
    end
    
    % Solves model with learning
    if model==2
        LEARNING=1; STD_AP=0.00049; STD_AT=0.001; RHO_T=0.001; RHO_P=0.999;ERISK=0;
    end
    
    % Solves model with learning, adds boost to target below
    if model==3
        LEARNING=1; STD_AP=0.00049; STD_AT=0.001; RHO_T=0.001; RHO_P=0.999;ERISK=0;
    end
    
    % Solves model with learning, adds boost to target below with rise in risk premia
    if model==4
        LEARNING=1; STD_AP=0.00049; STD_AT=0.001; RHO_T=0.001; RHO_P=0.999; ERISK=20;
    end
    save PARAM_EXTRA LEARNING STD_AP STD_AT RHO_T RHO_P ERISK
    
    
    
    extran=25;
    nperiods=7+extran;
    maxiter=10;
    
    
    irfshock = char('eps_c','eps_p','eps_t','eps_z'); % Shocks we look at: preference and pistar-shock
    
    % Baseline
    baseline=[           0         0     0    0% 2011Q1
                    -0.0650*1       0     0    0
                    -0.0650*1       0     0    0
                    -0.0650*1       0     0    0
                    -0.0650*1       0     0    0
                    -0.0650*1       0     0    0
                    0         0     0    0
                    zeros(extran,4)  ]*0.5;
    
    % This is the scenario shock
    scenario=baseline*0;
    
    scenario(8,2)=0.0025; % 2012Q4
    scenario(9,2)=0.00125; % 2013Q1
    scenario(10,2)=0.00125; % 2013Q2
    
    
    % ADDITIONAL BOOST
    if model>=3
        scenario(18:19,2)=0.0025; % 2014Q2
        scenario(24:25,2)=-0.0025; % 2015Q4
    end
    
    
    modnam_00 = 'dnk'; % base model (constraint 1 and 2 below don't bind)
    modnam_10 = 'dnk_zlb'; % first constraint is true
    modnam_01 = 'dnk_risk'; % second constraint is true
    modnam_11 = 'dnk_zlb_risk'; % both constraints bind
    
    
    % compile constraint
    constraint1 = 'r<-(1/BETA-1)';
    constraint_relax1 ='rnot>-(1/BETA-1)';
    constraint2 = 'ddp>0.01/100';
    constraint_relax2 ='ddp<0.01/100';
    
    
    
    if LEARNING==1
        
        paramfile_dnk
        
        FF = [ RHO_P 0
            0  RHO_T ];
        
        HH = [ (1-TAYLOR_R)*(1-TAYLOR_P) 1 ];
        
        RR = 0.000000000000000001  ;
        
        QQ = [ STD_AP^2  0
            0    STD_AT^2 ];
        
        [ scenario KK ] = recompute_learning_shock(scenario,irfshock,'paramfile_dnk',FF,HH,QQ,RR) ;
        
    end
    
    
    % First time we solve simulation only with baseline shocks
    [zdatabaseline_lin zdatabaseline_pie zdatass oobase_ Mbase_] = ...
        solve_two_constraints(...
        modnam_00,modnam_10,modnam_01,modnam_11,...
        constraint1, constraint2,...
        constraint_relax1, constraint_relax2,...
        baseline,irfshock,nperiods,0,maxiter);
    
    
    % % Second time we solve simulation with baseline shocks and scenario
    [zdatascenario_lin zdatascenario_pie zdatass oobase_ Mbase_] = ...
        solve_two_constraints(...
        modnam_00,modnam_10,modnam_01,modnam_11,...
        constraint1, constraint2,...
        constraint_relax1, constraint_relax2,...
        baseline+scenario,irfshock,nperiods,0,maxiter);
    
    
    
    
    % Note that we compute impulse responses in deviation from baseline
    % THE NO ZLB case is the linear case
    
    for i=1:Mbase_.endo_nbr
        eval([deblank(Mbase_.endo_names(i,:)),'_ss  = zdatass(i);']);
        eval([deblank(Mbase_.endo_names(i,:)),'_bl  = zdatabaseline_lin(:,i);']);
        eval([deblank(Mbase_.endo_names(i,:)),'_bl1 = zdatabaseline_lin(:,i)+zdatass(i);']);
        eval([deblank(Mbase_.endo_names(i,:)),'_bp  = zdatabaseline_pie(:,i);']);
        eval([deblank(Mbase_.endo_names(i,:)),'_bp1 = zdatabaseline_pie(:,i)+zdatass(i);']);
        eval([deblank(Mbase_.endo_names(i,:)),'_sl  = zdatascenario_lin(:,i);']);
        eval([deblank(Mbase_.endo_names(i,:)),'_sp  = zdatascenario_pie(:,i);']);
        eval([deblank(Mbase_.endo_names(i,:)),'_l   = zdatascenario_lin(:,i)-zdatabaseline_lin(:,i);']);
        eval([deblank(Mbase_.endo_names(i,:)),'_p   = zdatascenario_pie(:,i)-zdatabaseline_pie(:,i);']);
        eval([deblank(Mbase_.endo_names(i,:)),'_sp1  = zdatascenario_pie(:,i)+zdatass(i);']);
    end
    
    
    
    
    
    ylabels = char('','','','','','','','');
    
    % BASELINE
    legendlist = cellstr(char('ZLB binds'));
    figlabel = 'Baseline (All variables in % deviation from ss, except rates and B/Y)';
    titlelist = char('GDP','Policy Rate','Debt to GDP','Risk Premium',...
        '10-year Expected Inflation','Total Inflation','Perceived Inflation Target','Change in Inflation (ddp)');
    line_bl=100*[y_bl,4*r_bl1,b_bl1/4,4*rpremium_bl1,4*epten_bl1,4*dp_bl,4*dpp_bl,ddp_bl];
    line_bp=100*[y_bp,4*r_bp1,b_bp1/4,4*rpremium_bp1,4*epten_bp1,4*dp_bp,4*dpp_bp,ddp_bp];
    
    % SCENARIO 1
    legendlist = cellstr(char('ZLB binds'));
    figlabel = 'Scenario (All variables in % deviation from baseline, ddp from ss)';
    line_p=100*[y_p,4*r_p,b_p/4,4*rpremium_p,4*epten_p,4*dp_p,4*dpp_p,ddp_sp];
    line_l=100*[y_l,4*r_l,b_l/4,4*rpremium_l,4*epten_l,4*dp_l,4*dpp_l,ddp_sl];
    
    
    legendlist = cellstr(char('Baseline','Scenario'));
    figlabel = 'Scenario (All variables in levels or log levels)';
    line_b1=100*[y_bp1-y_bp1(1)+1,4*r_bp1,b_bp1/4,4*rpremium_bp1,4*epten_bp1,4*dp_bp,4*dpp_bp1,ddp_bp1];
    line_p1=100*[y_sp1-y_sp1(1)+1,4*r_sp1,b_sp1/4,4*rpremium_sp1,4*epten_sp1,4*dp_sp,4*dpp_sp1,ddp_sp1];
    
    paramfile_dnk
        
    line_p_story(:,:,model)=line_p;
    line_l_story(:,:,model)=line_l;
    
end






varselect=[1 7 6 5 2 ];
colore='b';
figlabel = '';
legendlist = cellstr(char('Perfect credibility','Perfect credibility, no ZLB'));
ylabels = char('% from baseline','ppt from baseline, AR','ppt from baseline, qoq, AR','ppt from baseline, AR','ppt from baseline, AR');
figure(9)
makechart99(titlelist(varselect,:),legendlist,colore,figlabel,baseyear+2000,ylabels,...
    line_p_story(:,varselect,1),line_l_story(:,varselect,1))
subplot(3,2,1)
ylim([-0.2 1.7]);
for m=2:5
    subplot(3,2,m)
    ylim([-0.5 2.4]);
end
addvplot5([2012.75 2015.25]);




varselect=[1 7 6 5 2 ];
colore='b';
figlabel = '';
legendlist = cellstr(char('Imperfect credibility','Imperfect credibility, no ZLB'));
ylabels = char('% from baseline','ppt from baseline, AR','ppt from baseline, qoq, AR','ppt from baseline, AR','ppt from baseline, AR');
figure(10)
makechart99(titlelist(varselect,:),legendlist,colore,figlabel,baseyear+2000,ylabels,...
    line_p_story(:,varselect,2),line_l_story(:,varselect,2))
subplot(3,2,1)
ylim([-0.2 1.7])
for m=2:5
    subplot(3,2,m)
    ylim([-0.5 2.4]);
end
addvplot5([2012.75 2015.25]);




figure(102)
varselect=[1 7 6 5 4 3];
colore='r';
figlabel = '';
legendlist = cellstr(char('Imperfect credibility','...with additional boost',...
    '...with boost and risk premia'));
ylabels = char('% from baseline','ppt from baseline, AR','ppt from baseline, qoq, AR','ppt from baseline, AR',...
    'ppt from baseline, AR','ppt from baseline');
makechart99bis(titlelist(varselect,:),legendlist,colore,figlabel,baseyear+2000,ylabels,...
    line_p_story(:,varselect,2),line_p_story(:,varselect,3),line_p_story(:,varselect,4))
subplot(3,2,5)
ylim([-1 5])
for m=2:4
    subplot(3,2,m)
    ylim([-0.2 2.5])
end
addvplot6([2012.75 2015.25]);



% Folder cleanup
delete *dynamic* *static* *results* *auxiliary_variables* *.log
